@@ -69,11 +69,11 @@ static int gpg_encrypt (GMimeCipherContext *ctx, gboolean sign,
 static int gpg_decrypt (GMimeCipherContext *ctx, GMimeStream *istream,
 			GMimeStream *ostream, GMimeException *ex);
 
-static int gpg_import (GMimeCipherContext *ctx, GMimeStream *istream,
-		       GMimeException *ex);
+static int gpg_import_keys (GMimeCipherContext *ctx, GMimeStream *istream,
+			    GMimeException *ex);
 
-static int gpg_export (GMimeCipherContext *ctx, GPtrArray *keys,
-		       GMimeStream *ostream, GMimeException *ex);
+static int gpg_export_keys (GMimeCipherContext *ctx, GPtrArray *keys,
+			    GMimeStream *ostream, GMimeException *ex);
 
 
 static GMimeCipherContextClass *parent_class = NULL;
@@ -120,8 +120,8 @@ g_mime_gpg_context_class_init (GMimeGpgContextClass *klass)
 	cipher_class->verify = gpg_verify;
 	cipher_class->encrypt = gpg_encrypt;
 	cipher_class->decrypt = gpg_decrypt;
-	cipher_class->import = gpg_import;
-	cipher_class->export = gpg_export;
+	cipher_class->import_keys = gpg_import_keys;
+	cipher_class->export_keys = gpg_export_keys;
 }
 
 static void
@@ -153,20 +153,20 @@ gpg_hash_id (GMimeCipherContext *ctx, const char *hash)
 	if (hash == NULL)
 		return GMIME_CIPHER_HASH_DEFAULT;
 	
-	if (!strcasecmp (hash, "pgp-"))
+	if (!g_strcasecmp (hash, "pgp-"))
 		hash += 4;
 	
-	if (!strcasecmp (hash, "md2"))
+	if (!g_strcasecmp (hash, "md2"))
 		return GMIME_CIPHER_HASH_MD2;
-	else if (!strcasecmp (hash, "md5"))
+	else if (!g_strcasecmp (hash, "md5"))
 		return GMIME_CIPHER_HASH_MD5;
-	else if (!strcasecmp (hash, "sha1"))
+	else if (!g_strcasecmp (hash, "sha1"))
 		return GMIME_CIPHER_HASH_SHA1;
-	else if (!strcasecmp (hash, "ripemd160"))
+	else if (!g_strcasecmp (hash, "ripemd160"))
 		return GMIME_CIPHER_HASH_RIPEMD160;
-	else if (!strcasecmp (hash, "tiger192"))
+	else if (!g_strcasecmp (hash, "tiger192"))
 		return GMIME_CIPHER_HASH_TIGER192;
-	else if (!strcasecmp (hash, "haval-5-160"))
+	else if (!g_strcasecmp (hash, "haval-5-160"))
 		return GMIME_CIPHER_HASH_HAVAL5160;
 	
 	return GMIME_CIPHER_HASH_DEFAULT;
@@ -223,9 +223,9 @@ struct _GpgCtx {
 	GPtrArray *recipients;
 	GMimeCipherHash hash;
 	
-	int stdin;
-	int stdout;
-	int stderr;
+	int stdin_fd;
+	int stdout_fd;
+	int stderr_fd;
 	int status_fd;
 	int passwd_fd;  /* only needed for sign/decrypt */
 	
@@ -285,9 +285,9 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->always_trust = FALSE;
 	gpg->armor = FALSE;
 	
-	gpg->stdin = -1;
-	gpg->stdout = -1;
-	gpg->stderr = -1;
+	gpg->stdin_fd = -1;
+	gpg->stdout_fd = -1;
+	gpg->stderr_fd = -1;
 	gpg->status_fd = -1;
 	gpg->passwd_fd = -1;
 	
@@ -340,7 +340,7 @@ gpg_ctx_set_userid (struct _GpgCtx *gpg, const char *userid)
 static void
 gpg_ctx_add_recipient (struct _GpgCtx *gpg, const char *keyid)
 {
-	if (gpg->mode != GPG_CTX_MODE_ENCRYPT)
+	if (gpg->mode != GPG_CTX_MODE_ENCRYPT && gpg->mode != GPG_CTX_MODE_EXPORT)
 		return;
 	
 	if (!gpg->recipients)
@@ -417,12 +417,12 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 		g_ptr_array_free (gpg->recipients, TRUE);
 	}
 	
-	if (gpg->stdin != -1)
-		close (gpg->stdin);
-	if (gpg->stdout != -1)
-		close (gpg->stdout);
-	if (gpg->stderr != -1)
-		close (gpg->stderr);
+	if (gpg->stdin_fd != -1)
+		close (gpg->stdin_fd);
+	if (gpg->stdout_fd != -1)
+		close (gpg->stdout_fd);
+	if (gpg->stderr_fd != -1)
+		close (gpg->stderr_fd);
 	if (gpg->status_fd != -1)
 		close (gpg->status_fd);
 	if (gpg->passwd_fd != -1)
@@ -511,6 +511,7 @@ gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, char **sfd, int passwd_fd,
 		break;
 	case GPG_CTX_MODE_VERIFY:
 		if (!g_mime_session_is_online (gpg->session)) {
+			/* this is a deprecated flag to gpg since 1.0.7 */
 			/*g_ptr_array_add (argv, "--no-auto-key-retrieve");*/
 			g_ptr_array_add (argv, "--keyserver-options");
 			g_ptr_array_add (argv, "no-auto-key-retrieve");
@@ -545,6 +546,17 @@ gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, char **sfd, int passwd_fd,
 		g_ptr_array_add (argv, "--output");
 		g_ptr_array_add (argv, "-");
 		break;
+	case GPG_CTX_MODE_IMPORT:
+		g_ptr_array_add (argv, "--import");
+		g_ptr_array_add (argv, "-");
+		break;
+	case GPG_CTX_MODE_EXPORT:
+		if (gpg->armor)
+			g_ptr_array_add (argv, "--armor");
+		g_ptr_array_add (argv, "--export");
+		for (i = 0; i < gpg->recipients->len; i++)
+			g_ptr_array_add (argv, gpg->recipients->pdata[i]);
+		break;
 	}
 	
 	for (i = 0; i < argv->len; i++)
@@ -560,12 +572,18 @@ static int
 gpg_ctx_op_start (struct _GpgCtx *gpg)
 {
 	char *status_fd = NULL, *passwd_fd = NULL;
-	int i, maxfd, fds[10];
+	int i, maxfd, errnosave, fds[10];
 	GPtrArray *argv;
+	struct stat st;
 	
 	for (i = 0; i < 10; i++)
 		fds[i] = -1;
 	
+	if (stat (gpg->path, &st) == -1)
+		goto exception;
+	
+	/* FIXME: check permissions on st.st_mode ? */
+	
 	maxfd = gpg->need_passwd ? 10 : 8;
 	for (i = 0; i < maxfd; i += 2) {
 		if (pipe (fds + i) == -1)
@@ -617,10 +635,10 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	
 	/* Parent */
 	close (fds[0]);
-	gpg->stdin = fds[1];
-	gpg->stdout = fds[2];
+	gpg->stdin_fd = fds[1];
+	gpg->stdout_fd = fds[2];
 	close (fds[3]);
-	gpg->stderr = fds[4];
+	gpg->stderr_fd = fds[4];
 	close (fds[5]);
 	gpg->status_fd = fds[6];
 	close (fds[7]);
@@ -630,20 +648,24 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 		fcntl (gpg->passwd_fd, F_SETFL, O_NONBLOCK);
 	}
 	
-	fcntl (gpg->stdin, F_SETFL, O_NONBLOCK);
-	fcntl (gpg->stdout, F_SETFL, O_NONBLOCK);
-	fcntl (gpg->stderr, F_SETFL, O_NONBLOCK);
+	fcntl (gpg->stdin_fd, F_SETFL, O_NONBLOCK);
+	fcntl (gpg->stdout_fd, F_SETFL, O_NONBLOCK);
+	fcntl (gpg->stderr_fd, F_SETFL, O_NONBLOCK);
 	fcntl (gpg->status_fd, F_SETFL, O_NONBLOCK);
 	
 	return 0;
 	
  exception:
 	
+	errnosave = errno;
+	
 	for (i = 0; i < 10; i++) {
 		if (fds[i] != -1)
 			close (fds[i]);
 	}
 	
+	errno = errnosave;
+	
 	return -1;
 }
 
@@ -793,7 +815,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GMimeException *ex)
 		switch (gpg->mode) {
 		case GPG_CTX_MODE_SIGN:
 			if (!strncmp (status, "SIG_CREATED ", 12)) {
-				gpg->complete = TRUE;
+				/* FIXME: save this state? */
 			}
 			break;
 		case GPG_CTX_MODE_VERIFY:
@@ -807,9 +829,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GMimeException *ex)
 					gpg->trust = GPG_TRUST_FULLY;
 				} else if (!strncmp (status, "ULTIMATE", 8)) {
 					gpg->trust = GPG_TRUST_ULTIMATE;
-				} 
-				
-				gpg->complete = TRUE;
+				}
 				
 				/* Since verifying a signature will never produce output
 				   on gpg's stdout descriptor, we use this EOF bit for
@@ -828,7 +848,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GMimeException *ex)
 			if (!strncmp (status, "BEGIN_ENCRYPTION", 16)) {
 				/* nothing to do... but we know to expect data on stdout soon */
 			} else if (!strncmp (status, "END_ENCRYPTION", 14)) {
-				gpg->complete = TRUE;
+				/* nothing to do, but we know the end is near? */
 			} else if (!strncmp (status, "NO_RECP", 7)) {
 				g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM,
 						      _("Failed to encrypt: No valid recipients specified."));
@@ -839,13 +859,17 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GMimeException *ex)
 			if (!strncmp (status, "BEGIN_DECRYPTION", 16)) {
 				/* nothing to do... but we know to expect data on stdout soon */
 			} else if (!strncmp (status, "END_DECRYPTION", 14)) {
-				gpg->complete = TRUE;
+				/* nothing to do, but we know the end is near? */
 			}
 			break;
+		case GPG_CTX_MODE_IMPORT:
+			/* hack to work around the fact that gpg
+                           doesn't write anything to stdout when
+                           importing keys */
+			if (!strncmp (status, "IMPORT_RES", 10))
+				gpg->seen_eof1 = TRUE;
+			break;
 		}
-		
-		if (gpg->complete)
-			d(printf ("okay, that's all folks...\n"));
 	}
 	
  recycle:
@@ -896,18 +920,18 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 	
  retry:
 	FD_ZERO (&rdset);
-	FD_SET (gpg->stdout, &rdset);
-	FD_SET (gpg->stderr, &rdset);
+	FD_SET (gpg->stdout_fd, &rdset);
+	FD_SET (gpg->stderr_fd, &rdset);
 	FD_SET (gpg->status_fd, &rdset);
 	
-	maxfd = MAX (gpg->stdout, gpg->stderr);
+	maxfd = MAX (gpg->stdout_fd, gpg->stderr_fd);
 	maxfd = MAX (maxfd, gpg->status_fd);
 	
-	if (gpg->stdin != -1 || gpg->passwd_fd != -1) {
+	if (gpg->stdin_fd != -1 || gpg->passwd_fd != -1) {
 		FD_ZERO (&wrset);
-		if (gpg->stdin != -1) {
-			FD_SET (gpg->stdin, &wrset);
-			maxfd = MAX (maxfd, gpg->stdin);
+		if (gpg->istream && gpg->stdin_fd != -1) {
+			FD_SET (gpg->stdin_fd, &wrset);
+			maxfd = MAX (maxfd, gpg->stdin_fd);
 		}
 		if (gpg->passwd_fd != -1) {
 			FD_SET (gpg->passwd_fd, &wrset);
@@ -953,7 +977,6 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 		
 		if (nread > 0) {
 			status_backup (gpg, buffer, nread);
-			
 			if (gpg_ctx_parse_status (gpg, ex) == -1)
 				return -1;
 		} else {
@@ -961,14 +984,14 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stdout, &rdset) && gpg->ostream) {
+	if (FD_ISSET (gpg->stdout_fd, &rdset) && gpg->ostream) {
 		char buffer[4096];
 		ssize_t nread;
 		
 		d(printf ("reading gpg's stdout...\n"));
 		
 		do {
-			nread = read (gpg->stdout, buffer, sizeof (buffer));
+			nread = read (gpg->stdout_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
 		if (nread == -1)
 			goto exception;
@@ -981,14 +1004,14 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stderr, &rdset)) {
+	if (FD_ISSET (gpg->stderr_fd, &rdset)) {
 		char buffer[4096];
 		ssize_t nread;
 		
 		d(printf ("reading gpg's stderr...\n"));
 		
 		do {
-			nread = read (gpg->stderr, buffer, sizeof (buffer));
+			nread = read (gpg->stderr_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
 		if (nread == -1)
 			goto exception;
@@ -1028,7 +1051,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 		gpg->send_passwd = FALSE;
 	}
 	
-	if (wrsetp && gpg->stdin != -1 && FD_ISSET (gpg->stdin, &wrset)) {
+	if (gpg->istream && wrsetp && gpg->stdin_fd != -1 && FD_ISSET (gpg->stdin_fd, &wrset)) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1041,7 +1064,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 			
 			do {
 				do {
-					w = write (gpg->stdin, buffer + nwritten, nread - nwritten);
+					w = write (gpg->stdin_fd, buffer + nwritten, nread - nwritten);
 				} while (w == -1 && (errno == EINTR || errno == EAGAIN));
 				
 				if (w > 0)
@@ -1053,8 +1076,8 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 		}
 		
 		if (g_mime_stream_eos (gpg->istream)) {
-			close (gpg->stdin);
-			gpg->stdin = -1;
+			close (gpg->stdin_fd);
+			gpg->stdin_fd = -1;
 		}
 	}
 	
@@ -1075,6 +1098,12 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 	case GPG_CTX_MODE_DECRYPT:
 		mode = "decrypt";
 		break;
+	case GPG_CTX_MODE_IMPORT:
+		mode = "import keys";
+		break;
+	case GPG_CTX_MODE_EXPORT:
+		mode = "export keys";
+		break;
 	default:
 		g_assert_not_reached ();
 		mode = NULL;
@@ -1083,13 +1112,13 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GMimeException *ex)
 	
 	if (gpg->diagnostics->len) {
 		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
-				       _("Failed to GPG %s message: %s\n\n%.*s"),
+				       _("Failed to GPG %s: %s\n\n%.*s"),
 				       mode, g_strerror (errno),
 				       gpg->diagnostics->len,
 				       gpg->diagnostics->data);
 	} else {
 		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
-				       _("Failed to GPG %s message: %s\n"),
+				       _("Failed to GPG %s: %s\n"),
 				       mode, g_strerror (errno));
 	}
 	
@@ -1194,8 +1223,9 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM,
-				      _("Failed to execute gpg."));
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1292,8 +1322,9 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM,
-				      _("Failed to execute gpg."));
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
 		goto exception;
 	}
@@ -1359,8 +1390,9 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 	}
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM,
-				      _("Failed to execute gpg."));
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1406,8 +1438,9 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 	gpg_ctx_set_ostream (gpg, ostream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
-		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM,
-				      _("Failed to execute gpg."));
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
 		gpg_ctx_free (gpg);
 		
 		return -1;
@@ -1440,15 +1473,99 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 }
 
 static int
-gpg_import (GMimeCipherContext *ctx, GMimeStream *istream, GMimeException *ex)
+gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GMimeException *ex)
 {
-	return -1;
+	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
+	struct _GpgCtx *gpg;
+	
+	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_IMPORT);
+	gpg_ctx_set_istream (gpg, istream);
+	
+	if (gpg_ctx_op_start (gpg) == -1) {
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
+		gpg_ctx_free (gpg);
+		
+		return -1;
+	}
+	
+	while (!gpg_ctx_op_complete (gpg)) {
+		if (gpg_ctx_op_step (gpg, ex) == -1) {
+			gpg_ctx_op_cancel (gpg);
+			gpg_ctx_free (gpg);
+			
+			return -1;
+		}
+	}
+	
+	if (gpg_ctx_op_wait (gpg) != 0) {
+		char *diagnostics;
+		
+		diagnostics = gpg_ctx_get_diagnostics (gpg);
+		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM, diagnostics);
+		g_free (diagnostics);
+		
+		gpg_ctx_free (gpg);
+		
+		return -1;
+	}
+	
+	gpg_ctx_free (gpg);
+	
+	return 0;
 }
 
 static int
-gpg_export (GMimeCipherContext *ctx, GPtrArray *keys, GMimeStream *ostream, GMimeException *ex)
+gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostream, GMimeException *ex)
 {
-	return -1;
+	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
+	struct _GpgCtx *gpg;
+	int i;
+	
+	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_EXPORT);
+	gpg_ctx_set_armor (gpg, TRUE);
+	gpg_ctx_set_ostream (gpg, ostream);
+	
+	for (i = 0; i < keys->len; i++) {
+		gpg_ctx_add_recipient (gpg, keys->pdata[i]);
+	}
+	
+	if (gpg_ctx_op_start (gpg) == -1) {
+		g_mime_exception_setv (ex, GMIME_EXCEPTION_SYSTEM,
+				       _("Failed to execute gpg: %s"),
+				       errno ? g_strerror (errno) : _("Unknown"));
+		gpg_ctx_free (gpg);
+		
+		return -1;
+	}
+	
+	while (!gpg_ctx_op_complete (gpg)) {
+		if (gpg_ctx_op_step (gpg, ex) == -1) {
+			gpg_ctx_op_cancel (gpg);
+			gpg_ctx_free (gpg);
+			
+			return -1;
+		}
+	}
+	
+	if (gpg_ctx_op_wait (gpg) != 0) {
+		char *diagnostics;
+		
+		diagnostics = gpg_ctx_get_diagnostics (gpg);
+		g_mime_exception_set (ex, GMIME_EXCEPTION_SYSTEM, diagnostics);
+		g_free (diagnostics);
+		
+		gpg_ctx_free (gpg);
+		
+		return -1;
+	}
+	
+	gpg_ctx_free (gpg);
+	
+	return 0;
 }
 
 
@@ -1481,10 +1598,27 @@ g_mime_gpg_context_new (GMimeSession *session, const char *path)
 }
 
 
+/**
+ * g_mime_gpg_context_get_always_trust:
+ * @ctx: gpg context
+ *
+ * Gets the @always_trust flag on the gpg context.
+ *
+ * Returns the @always_trust flag on the gpg context.
+ **/
+gboolean
+g_mime_gpg_context_get_always_trust (GMimeGpgContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_GPG_CONTEXT (ctx), FALSE);
+	
+	return ctx->always_trust;
+}
+
+
 /**
  * g_mime_gpg_context_set_always_trust:
  * @ctx: gpg context
- * @always_trust always truct flag
+ * @always_trust: always truct flag
  *
  * Sets the @always_trust flag on the gpg context which is used for
  * encryption.
